/*
**
**                           Main.c
**
**
**********************************************************************/
/*
   Last committed:     $Revision: 00 $
   Last changed by:    $Author: $
   Last changed date:  $Date:  $
   ID:                 $Id:  $

**********************************************************************/
#include "stm32f10x_conf.h"
#include "system_stm32f10x.h"


#include <string.h>
#include <stdio.h>
#include <stdlib.h>




#ifndef NULL
#define NULL (void *) 0
#endif

#define LED_PORT                    GPIOC
#define LED_D7                      ( 0xFFFF & ~(1<<0) )
#define LED_D9                      ( 0xFFFF & ~(1<<1) )
#define LED_D12                     ( 0xFFFF & ~(1<<14))
#define LED_D13                     ( 0xFFFF & ~(1<<15))

#define GPIO_TOGGLE(PORT, BIT)      (PORT->ODR ^= (1<<BIT))

/* can settings */
#define CAN_NETWORK_MANAGMENT_ID   0x0000
#define CAN_SYNC_MSG_ID            0x0080

#define CAN_MASTER_HEARTBEAT_ID    0x0700
#define CAN_SLAVE_HEARTBEAT_ID     0x0701
enum can_slave_heartbeat{
    BOOT_UP         = 0x00,
    OFF_BUS         = 0x01,
    STOPPED         = 0x04,
    OPERATIONAL     = 0x05,
    PRE_OPERATIONAL = 0x7F,
};

#define CAN_EMGY_MSG_ID            0x0083
enum can_emgcy{
     NO                   =  0x00,
     UNDEFINED_TYPE       =  0x10,
     CURRENT              =  0x20,
     VOLTAGE              =  0x30,
     TEMPERATURE          =  0x40,
     HARDWARE             =  0x50,
     SOFTWARE             =  0x60,
     ADDITIONALS_MODULES  =  0x70,
     COMMUNICATION        =  0x80,
     EXTERN               =  0x90,
     DEVICE_SPECIFIC      =  0xFF,
};


#define CAN_TX_PDO1                 0x0183
#define CAN_RX_PDO0                 0x0203
#define CAN_TX_PDO2                 0x0283
#define CAN_RX_PDO2                 0x0303
#define CAN_TX_PDO3                 0x0383
#define CAN_RX_PDO3                 0x0403
#define CAN_TX_PDO4                 0x0483
#define CAN_RX_PDO4                 0x0503
#define CAN_TX_SDO                  0x0583
#define CAN_RX_SDO                  0x0603
#define CAN_NODE_GUARD              0x0703

typedef enum bitrate_ty {
   _500_kbit,
   _250_kbit,
   _125_kbit,
   _100_kbit,
    _95_kbit,
    _83_kbit,
    _50_kbit,
    _33_kbit,
    _20_kbit,
    _10_kbit,

    COUNT_BITRATE,
} enBitrate_t;


typedef struct Bitrate_t {
    unsigned int Prescaler;
    unsigned char BS1;
    unsigned char BS2;
} Bitrate_st ;

/* system clock = 72MHz (PCLK1 = 36 MHz) */
const Bitrate_st stBitrate[COUNT_BITRATE]= {
//    PRESCal       BS1          BS2
      { 6     , CAN_BS1_8tq, CAN_BS2_3tq},    // 500kbit
      { 12    , CAN_BS1_8tq, CAN_BS2_3tq},    // 250kbit
      { 24    , CAN_BS1_8tq, CAN_BS2_3tq},    // 125kbit
      { 30    , CAN_BS1_8tq, CAN_BS2_3tq},    // 100kbit
      { 42    , CAN_BS1_6tq, CAN_BS2_2tq},    // 95kbit
      { 36    , CAN_BS1_8tq, CAN_BS2_3tq},    // 83kbit
      { 60    , CAN_BS1_8tq, CAN_BS2_3tq},    // 50kbit
      { 90    , CAN_BS1_8tq, CAN_BS2_3tq},    // 33kbit
      { 150   , CAN_BS1_8tq, CAN_BS2_3tq},    // 20kbit
      { 300   , CAN_BS1_8tq, CAN_BS2_3tq},    // 10kbit
};


void _canInit(void)
{
    GPIO_InitTypeDef gpiox;
    CAN_InitTypeDef canx;

    RCC_APB1PeriphClockCmd(RCC_APB1Periph_CAN1, ENABLE);
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB | RCC_APB2Periph_AFIO, ENABLE);

    gpiox.GPIO_Pin      = GPIO_Pin_8;
    gpiox.GPIO_Mode     = GPIO_Mode_IPU;
    gpiox.GPIO_Speed    = GPIO_Speed_10MHz;
    GPIO_Init(GPIOB, &gpiox);

    gpiox.GPIO_Pin      = GPIO_Pin_9;
    gpiox.GPIO_Mode     = GPIO_Mode_AF_PP;
    gpiox.GPIO_Speed    = GPIO_Speed_10MHz;
    GPIO_Init(GPIOB, &gpiox);

    GPIO_PinRemapConfig(GPIO_Remap1_CAN1 , ENABLE);

    CAN_DeInit(CAN1);

    CAN_StructInit(&canx);

    // Konfiguriert die CAN Baudrate fr eine 72MHz Clock
    canx.CAN_SJW        = CAN_SJW_1tq;  // SJW 1-4
    canx.CAN_BS1        = stBitrate[_20_kbit].BS1;  // Samplepoint 72%
    canx.CAN_BS2        = stBitrate[_20_kbit].BS2;  // Samplepoint 72%
    canx.CAN_Prescaler  = stBitrate[_20_kbit].Prescaler;


    CAN_Init(CAN1, &canx);
}

void _uartInit(void)
{
    USART_InitTypeDef usartx;
    GPIO_InitTypeDef  gpiox;

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO | RCC_APB2Periph_GPIOA, ENABLE);
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);

    gpiox.GPIO_Pin      = GPIO_Pin_2;
    gpiox.GPIO_Speed    = GPIO_Speed_10MHz;
    gpiox.GPIO_Mode     = GPIO_Mode_AF_PP;
    GPIO_Init(GPIOA, &gpiox);

    gpiox.GPIO_Pin      = GPIO_Pin_3;
    gpiox.GPIO_Speed    = GPIO_Speed_10MHz;
    gpiox.GPIO_Mode     = GPIO_Mode_IN_FLOATING;
    GPIO_Init(GPIOA, &gpiox);


    usartx.USART_BaudRate               = 9600;
    usartx.USART_WordLength             = USART_WordLength_8b;
    usartx.USART_StopBits               = USART_StopBits_1;
    usartx.USART_Parity                 = USART_Parity_No ;
    usartx.USART_Mode                   = USART_Mode_Rx | USART_Mode_Tx;
    usartx.USART_HardwareFlowControl    = USART_HardwareFlowControl_None;

    USART_Init(USART2, &usartx);

    USART_Cmd(USART2, ENABLE);
}

void _uartChar(char c, USART_TypeDef *uart)
{
    while ((uart->SR & USART_SR_TXE) == 0);
    uart->DR = c;
}

void _uartStr(char *str, USART_TypeDef *uart)
{
    while(*str)
    {
        while ((uart->SR & USART_SR_TXE) == 0);
        _uartChar(*str++,uart);
    }
}

void _canSendMsgBuff(char msg[],CanTxMsg *canBuff)
{
    uint8_t buffCnt = 0x00;
    if(msg != NULL)
    {
         while(buffCnt < 8)
        {
            canBuff->Data[buffCnt]=msg[buffCnt];
            buffCnt++;
        }
    }
    CAN_Transmit(CAN1,canBuff);
}

void _canSlaveHeartbeat(CanTxMsg *canBuff,enum can_slave_heartbeat state)
{
    canBuff->StdId = CAN_SLAVE_HEARTBEAT_ID;
    canBuff->ExtId = 0;
    canBuff->RTR = CAN_RTR_DATA;
    canBuff->IDE = CAN_ID_STD;
    canBuff->DLC = 1;
    canBuff->Data[0] = state;
    _canSendMsgBuff(NULL,canBuff);
}

void _ledInit(void)
{
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC,ENABLE);
    GPIO_InitTypeDef ledPort;

    ledPort.GPIO_Mode   = GPIO_Mode_Out_PP;
    ledPort.GPIO_Pin    = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_14 | GPIO_Pin_15;
    ledPort.GPIO_Speed  = GPIO_Speed_50MHz;

    GPIO_Init(GPIOC,&ledPort);
    GPIO_Write(GPIOC,0xFFFF);
}

void SysTick_Handler(void)
{
    static uint8_t bitState = 0x00;

    if(bitState == 0x01)
    {
        GPIO_WriteBit(LED_PORT,GPIO_Pin_0,Bit_RESET);
    }
    if(bitState == 0x02)
    {
        GPIO_WriteBit(LED_PORT,GPIO_Pin_1,Bit_RESET);
    }
    if(bitState == 0x03)
    {
        GPIO_WriteBit(LED_PORT,GPIO_Pin_14,Bit_RESET);
    }
    if(bitState == 0x04)
    {
        GPIO_WriteBit(LED_PORT,GPIO_Pin_15,Bit_RESET);
    }
    if(bitState == 0x05)
    {
        GPIO_WriteBit(LED_PORT,GPIO_Pin_0,Bit_SET);
        GPIO_WriteBit(LED_PORT,GPIO_Pin_1,Bit_SET);
        GPIO_WriteBit(LED_PORT,GPIO_Pin_14,Bit_SET);
        GPIO_WriteBit(LED_PORT,GPIO_Pin_15,Bit_SET);
    }
    if(bitState == 0x06)
    {
        GPIO_WriteBit(LED_PORT,GPIO_Pin_15,Bit_RESET);
    }
    if(bitState == 0x07)
    {
        GPIO_WriteBit(LED_PORT,GPIO_Pin_14,Bit_RESET);
    }
    if(bitState == 0x08)
    {
        GPIO_WriteBit(LED_PORT,GPIO_Pin_1,Bit_RESET);
    }
    if(bitState == 0x09)
    {
        GPIO_WriteBit(LED_PORT,GPIO_Pin_0,Bit_RESET);
    }
    if(bitState == 0x0A)
    {
        GPIO_WriteBit(LED_PORT,GPIO_Pin_0,Bit_SET);
        GPIO_WriteBit(LED_PORT,GPIO_Pin_1,Bit_SET);
        GPIO_WriteBit(LED_PORT,GPIO_Pin_14,Bit_SET);
        GPIO_WriteBit(LED_PORT,GPIO_Pin_15,Bit_SET);
        bitState = 0x00;
    }
    bitState++;
}


int main(void)
{

    SystemInit();

    //SysTick_Config(SystemCoreClock / 10);
    //_uartInit();
    //_ledInit();
    _canInit();

    //_uartStr("****************************\r\n",USART2);
    //_uartStr("*  STM32F105R8T6 - CAN Board\r\n",USART2);
    //_uartStr("****************************\r\n",USART2);

  while(1)
  {
        CanTxMsg canMessage;

        canMessage.StdId = 0x123;
        canMessage.ExtId = 0;
        canMessage.RTR = CAN_RTR_DATA;
        canMessage.IDE = CAN_ID_STD;
        canMessage.DLC = 8;

        canMessage.Data[0] = 0;
        canMessage.Data[1] = 1;
        canMessage.Data[2] = 2;
        canMessage.Data[3] = 3;
        canMessage.Data[4] = 4;
        canMessage.Data[5] = 5;
        canMessage.Data[6] = 6;
        canMessage.Data[7] = 7;

        CAN_Transmit(CAN1, &canMessage);
  }
}
